@@ -7,8 +7,8 @@ import time
 import warnings
 from sentinels import NOTHING
 from .filtering import filter_applies, resolve_key_value
-from . import ObjectId, OperationFailure
-from .helpers import basestring
+from . import ObjectId, OperationFailure, DuplicateKeyError
+from .helpers import basestring, xrange
 
 try:
     # Optional requirements for providing Map-Reduce functionality
@@ -22,9 +22,11 @@ except ImportError:
     json_utils = SON = None
 
 from six import (
-                 iteritems,
-                 itervalues,
-                 iterkeys)
+    string_types,
+    text_type,
+    iteritems,
+    itervalues,
+    iterkeys)
 from mongomock import helpers
 
 
@@ -38,10 +40,10 @@ class Collection(object):
 
     def __repr__(self):
         return "Collection({0}, '{1}')".format(self._Collection__database, self.name)
-    
+
     def __getitem__(self, name):
         return self._Collection__database[self.name + '.' + name]
-    
+
     def __getattr__(self, name):
         return self.__getitem__(name)
 
@@ -51,6 +53,10 @@ class Collection(object):
         return self._insert(data)
 
     def _insert(self, data):
+
+        if not all(isinstance(k, string_types) for k in data):
+            raise ValueError("Document keys must be strings")
+
         if '_id' not in data:
             data['_id'] = ObjectId()
         object_id = data['_id']
@@ -138,6 +144,8 @@ class Collection(object):
                     for field, value in iteritems(v):
                         nested_field_list = field.rsplit('.')
                         if len(nested_field_list) == 1:
+                            if field not in existing_document:
+                                existing_document[field] = []
                             # document should be a list
                             # append to it
                             if isinstance(value, dict):
@@ -183,7 +191,7 @@ class Collection(object):
                         existing_document.clear()
                         if _id:
                             existing_document['_id'] = _id
-                        existing_document.update(document)
+                        existing_document.update(self._internalize_dict(document))
                         if existing_document['_id'] != _id:
                             # id changed, fix index
                             del self._documents[_id]
@@ -197,11 +205,11 @@ class Collection(object):
                 break
 
         return {
-            u"connectionId": self._Collection__database.connection._id,
-            u"err": None,
-            u"ok": 1.0,
-            u"n": num_updated,
-            u"updatedExisting": updated_existing,
+            text_type("connectionId"): self._Collection__database.connection._id,
+            text_type("err"): None,
+            text_type("ok"): 1.0,
+            text_type("n"): num_updated,
+            text_type("updatedExisting"): updated_existing,
         }
 
     def _get_subdocument(self, existing_document, spec, nested_field_list):
@@ -245,20 +253,27 @@ class Collection(object):
         # TODO: this looks a little too naive...
         return dict((k, v) for k, v in iteritems(doc) if not k.startswith("$"))
 
-    def find(self, spec = None, fields = None, filter = None, sort = None, timeout = True, limit = 0, snapshot = False, as_class = None):
+    def find(self, spec = None, fields = None, filter = None, sort = None, timeout = True, limit = 0, snapshot = False, as_class = None, skip = 0):
         if filter is not None:
             _print_deprecation_warning('filter', 'spec')
             if spec is None:
                 spec = filter
         if as_class is None:
             as_class = dict
-        return Cursor(self, functools.partial(self._get_dataset, spec, sort, fields, as_class), limit=limit)
+        return Cursor(self, functools.partial(self._get_dataset, spec, sort, fields, as_class, skip), limit=limit)
 
-    def _get_dataset(self, spec, sort, fields, as_class):
+    def _get_dataset(self, spec, sort, fields, as_class, skip):
         dataset = (self._copy_only_fields(document, fields, as_class) for document in self._iter_documents(spec))
         if sort:
             for sortKey, sortDirection in reversed(sort):
-                dataset = iter(sorted(dataset, key = lambda x: x[sortKey], reverse = sortDirection < 0))
+                dataset = iter(sorted(dataset, key = lambda x: resolve_key_value(sortKey, x), reverse = sortDirection < 0))
+
+        for i in xrange(skip):
+            try:
+                unused = next(dataset)
+            except StopIteration:
+                pass
+
         return dataset
 
     def _copy_field(self, obj, container):
@@ -570,9 +585,9 @@ class Cursor(object):
             direction = 1
         if isinstance(key_or_list, (tuple, list)):
             for sortKey, sortDirection in reversed(key_or_list):
-                self._dataset = iter(sorted(self._dataset, key = lambda x: x[sortKey], reverse = sortDirection < 0))
+                self._dataset = iter(sorted(self._dataset, key = lambda x:resolve_key_value(sortKey, x), reverse = sortDirection < 0))
         else:
-            self._dataset = iter(sorted(self._dataset, key = lambda x:x[key_or_list], reverse = direction < 0))
+            self._dataset = iter(sorted(self._dataset, key = lambda x:resolve_key_value(key_or_list, x), reverse = direction < 0))
         return self
     def count(self):
         arr = [x for x in self._dataset]
