@@ -104,14 +104,17 @@ class _PymongoConnectionMixin(object):
         return mongomock.Connection
 
 class _CollectionTest(_CollectionComparisonTest):
+
     def test__find_is_empty(self):
         self.cmp.do.remove()
         self.cmp.compare.find()
+
     def test__inserting(self):
         self.cmp.do.remove()
         data = {"a" : 1, "b" : 2, "c" : "data"}
         self.cmp.do.insert(data)
         self.cmp.compare.find() # single document, no need to ignore order
+
     def test__bulk_insert(self):
         objs = [{"a" : 2, "b" : {"c" : 3}}, {"c" : 5}, {"d" : 7}]
         results_dict = self.cmp.do.insert(objs)
@@ -120,7 +123,6 @@ class _CollectionTest(_CollectionComparisonTest):
             self.assertEquals(len(set(results)), len(results), "Returned object ids not unique!")
         self.cmp.compare_ignore_order.find()
 
-
     def test__save(self):
         self.cmp.do.insert({"_id" : "b"}) #add an item with a non ObjectId _id first.
         self.cmp.do.save({"_id":ObjectId(), "someProp":1}, safe=True)
@@ -199,7 +201,7 @@ class _CollectionTest(_CollectionComparisonTest):
         self.cmp.compare.find({'hat': 'a hat'})
         self.cmp.compare.find({'hat.color.cat': 'red'})
 
-    def test__find_by_regex(self):
+    def test__find_by_regex_object(self):
         """Test searching with regular expression objects."""
         bob = {'name': 'bob'}
         sam = {'name': 'sam'}
@@ -211,6 +213,47 @@ class _CollectionTest(_CollectionComparisonTest):
         regex = re.compile('bob|notsam')
         self.cmp.compare_ignore_order.find({"name" : regex})
 
+    def test__find_by_regex_string(self):
+        """Test searching with regular expression string."""
+        bob = {'name': 'bob'}
+        sam = {'name': 'sam'}
+        self.cmp.do.insert(bob)
+        self.cmp.do.insert(sam)
+        self.cmp.compare_ignore_order.find()
+        self.cmp.compare_ignore_order.find({"name": {'$regex': 'bob|sam'}})
+        self.cmp.compare_ignore_order.find({'name': {'$regex': 'bob|notsam'}})
+
+    def test__find_in_array_by_regex_object(self):
+        """Test searching inside array with regular expression object."""
+        bob = {'name': 'bob', 'text': ['abcd', 'cde']}
+        sam = {'name': 'sam', 'text': ['bde']}
+        self.cmp.do.insert(bob)
+        self.cmp.do.insert(sam)
+        regex = re.compile('^a')
+        self.cmp.compare_ignore_order.find({"text": regex})
+        regex = re.compile('e$')
+        self.cmp.compare_ignore_order.find({"text": regex})
+        regex = re.compile('bde|cde')
+        self.cmp.compare_ignore_order.find({"text": regex})
+
+    def test__find_in_array_by_regex_string(self):
+        """Test searching inside array with regular expression string"""
+        bob = {'name': 'bob', 'text': ['abcd', 'cde']}
+        sam = {'name': 'sam', 'text': ['bde']}
+        self.cmp.do.insert(bob)
+        self.cmp.do.insert(sam)
+        self.cmp.compare_ignore_order.find({"text": {'$regex': '^a'}})
+        self.cmp.compare_ignore_order.find({"text": {'$regex': 'e$'}})
+        self.cmp.compare_ignore_order.find({"text": {'$regex': 'bcd|cde'}})
+
+    def test__find_by_regex_string_on_absent_field_dont_break(self):
+        """Test searching on absent field with regular expression string dont break"""
+        bob = {'name': 'bob'}
+        sam = {'name': 'sam'}
+        self.cmp.do.insert(bob)
+        self.cmp.do.insert(sam)
+        self.cmp.compare_ignore_order.find({"text": {'$regex': 'bob|sam'}})
+
     def test__find_by_elemMatch(self):
         self.cmp.do.insert({"field": [{"a": 1, "b": 2}, {"c": 3, "d": 4}]})
         self.cmp.do.insert({"field": [{"a": 1, "b": 4}, {"c": 3, "d": 8}]})
@@ -293,6 +336,26 @@ class _CollectionTest(_CollectionComparisonTest):
         self.cmp.compare.find(sort = [("b", 1), ("a", -1)])
         self.cmp.compare.find(sort = [("b", 1), ("a", -1), ("c", 1)])
 
+    def test__find_sort_list_nested_doc(self):
+        self.cmp.do.remove()
+        for data in ({"root": {"a" : 1, "b" : 3, "c" : "data1"}},
+                     {"root": {"a" : 2, "b" : 2, "c" : "data3"}},
+                     {"root": {"a" : 3, "b" : 1, "c" : "data2"}}):
+            self.cmp.do.insert(data)
+        self.cmp.compare.find(sort = [("root.a", 1), ("root.b", -1)])
+        self.cmp.compare.find(sort = [("root.b", 1), ("root.a", -1)])
+        self.cmp.compare.find(sort = [("root.b", 1), ("root.a", -1), ("root.c", 1)])
+
+    def test__find_sort_list_nested_list(self):
+        self.cmp.do.remove()
+        for data in ({"root": [{"a" : 1, "b" : 3, "c" : "data1"}]},
+                     {"root": [{"a" : 2, "b" : 2, "c" : "data3"}]},
+                     {"root": [{"a" : 3, "b" : 1, "c" : "data2"}]}):
+            self.cmp.do.insert(data)
+        self.cmp.compare.find(sort = [("root.0.a", 1), ("root.0.b", -1)])
+        self.cmp.compare.find(sort = [("root.0.b", 1), ("root.0.a", -1)])
+        self.cmp.compare.find(sort = [("root.0.b", 1), ("root.0.a", -1), ("root.0.c", 1)])
+
     def test__find_limit(self):
         self.cmp.do.remove()
         for data in ({"a" : 1, "b" : 3, "c" : "data1"},
@@ -491,6 +554,18 @@ class _CollectionTest(_CollectionComparisonTest):
         self.cmp.do.update({'hat': {'$elemMatch': {'name': 'derby'}}}, {'$push': {'hat.$.sizes': {'$each': ['M', 'S']}}})
         self.cmp.compare.find({'name': 'bob'})
 
+    def test__push_to_absent_field(self):
+        self.cmp.do.remove()
+        self.cmp.do.insert({'name': 'bob'})
+        self.cmp.do.update({'name': 'bob'}, {'$push': {'hat': 'wide'}})
+        self.cmp.compare.find({'name': 'bob'})
+
+    def test__push_each_to_absent_field(self):
+        self.cmp.do.remove()
+        self.cmp.do.insert({'name': 'bob'})
+        self.cmp.do.update({'name': 'bob'}, {'$push': {'hat': {'$each': ['wide', 'blue']}}})
+        self.cmp.compare.find({'name': 'bob'})
+
     def test__drop(self):
         data = {'a': 1}
         self.cmp.do.insert({"name" : "another new"})
@@ -626,6 +701,24 @@ class _SortSkipLimitTest(_CollectionComparisonTest):
         self.cmp.compare(_SORT("a")).find()
         self.cmp.compare(_SORT("b")).find()
 
+    def test__sort_name_nested_doc(self):
+        self.cmp.do.remove()
+        for data in ({"root": {"a" : 1, "b" : 3, "c" : "data1"}},
+                     {"root": {"a" : 2, "b" : 2, "c" : "data3"}},
+                     {"root": {"a" : 3, "b" : 1, "c" : "data2"}}):
+            self.cmp.do.insert(data)
+        self.cmp.compare(_SORT("root.a")).find()
+        self.cmp.compare(_SORT("root.b")).find()
+
+    def test__sort_name_nested_list(self):
+        self.cmp.do.remove()
+        for data in ({"root": [{"a" : 1, "b" : 3, "c" : "data1"}]},
+                     {"root": [{"a" : 2, "b" : 2, "c" : "data3"}]},
+                     {"root": [{"a" : 3, "b" : 1, "c" : "data2"}]}):
+            self.cmp.do.insert(data)
+        self.cmp.compare(_SORT("root.0.a")).find()
+        self.cmp.compare(_SORT("root.0.b")).find()
+
     def test__sort_list(self):
         self.cmp.do.remove()
         for data in ({"a" : 1, "b" : 3, "c" : "data1"},
@@ -636,6 +729,26 @@ class _SortSkipLimitTest(_CollectionComparisonTest):
         self.cmp.compare(_SORT([("b", 1), ("a", -1)])).find()
         self.cmp.compare(_SORT([("b", 1), ("a", -1), ("c", 1)])).find()
 
+    def test__sort_list_nested_doc(self):
+        self.cmp.do.remove()
+        for data in ({"root": {"a" : 1, "b" : 3, "c" : "data1"}},
+                     {"root": {"a" : 2, "b" : 2, "c" : "data3"}},
+                     {"root": {"a" : 3, "b" : 1, "c" : "data2"}}):
+            self.cmp.do.insert(data)
+        self.cmp.compare(_SORT([("root.a", 1), ("root.b", -1)])).find()
+        self.cmp.compare(_SORT([("root.b", 1), ("root.a", -1)])).find()
+        self.cmp.compare(_SORT([("root.b", 1), ("root.a", -1), ("root.c", 1)])).find()
+
+    def test__sort_list_nested_list(self):
+        self.cmp.do.remove()
+        for data in ({"root": [{"a" : 1, "b" : 3, "c" : "data1"}]},
+                     {"root": [{"a" : 2, "b" : 2, "c" : "data3"}]},
+                     {"root": [{"a" : 3, "b" : 1, "c" : "data2"}]}):
+            self.cmp.do.insert(data)
+        self.cmp.compare(_SORT([("root.0.a", 1), ("root.0.b", -1)])).find()
+        self.cmp.compare(_SORT([("root.0.b", 1), ("root.0.a", -1)])).find()
+        self.cmp.compare(_SORT([("root.0.b", 1), ("root.0.a", -1), ("root.0.c", 1)])).find()
+
     def test__close(self):
         # Does nothing - just make sure it exists and takes the right args
         self.cmp.do(lambda cursor: cursor.close()).find()
